/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.remote;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;

import net.micode.notes.R;
import net.micode.notes.ui.NotesListActivity;
import net.micode.notes.ui.NotesPreferenceActivity;

/**
 * GTaskASyncTask类继承自AsyncTask，用于在后台执行Google任务同步操作，
 * 并在同步过程中更新通知，同时支持取消同步操作。
 */
public class GTaskASyncTask extends AsyncTask<Void, String, Integer> {

    // 通知的唯一标识符
    private static int GTASK_SYNC_NOTIFICATION_ID = 5234235;

    /**
     * 定义同步完成的监听器接口
     */
    public interface OnCompleteListener {
        /**
         * 同步完成时调用的方法
         */
        void onComplete();
    }

    // 上下文对象
    private Context mContext;
    // 通知管理器
    private NotificationManager mNotifiManager;
    // Google任务管理器实例
    private GTaskManager mTaskManager;
    // 同步完成监听器
    private OnCompleteListener mOnCompleteListener;

    /**
     * 构造函数，初始化任务所需的上下文、监听器、通知管理器和任务管理器。
     *
     * @param context 上下文对象
     * @param listener 同步完成监听器
     */
    public GTaskASyncTask(Context context, OnCompleteListener listener) {
        mContext = context;
        mOnCompleteListener = listener;
        // 获取通知服务
        mNotifiManager = (NotificationManager) mContext
                .getSystemService(Context.NOTIFICATION_SERVICE);
        // 获取GTaskManager实例
        mTaskManager = GTaskManager.getInstance();
    }

    /**
     * 取消同步操作
     */
    public void cancelSync() {
        mTaskManager.cancelSync();
    }

    /**
     * 发布同步进度消息
     *
     * @param message 进度消息
     */
    public void publishProgess(String message) {
        publishProgress(new String[]{message});
    }

    /**
     * 显示通知
     *
     * @param tickerId 通知的提示信息资源ID
     * @param content  通知的内容
     */
    private void showNotification(int tickerId, String content) {
        PendingIntent pendingIntent;
        // 根据tickerId设置不同的PendingIntent
        if (tickerId != R.string.ticker_success) {
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext,
                    NotesPreferenceActivity.class), 0);
        } else {
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext,
                    NotesListActivity.class), 0);
        }

        // 构建通知
        Notification.Builder builder = new Notification.Builder(mContext)
                .setAutoCancel(true)
                .setContentTitle("title")
                .setContentText("text")
                .setContentIntent(pendingIntent)
                .setWhen(System.currentTimeMillis())
                .setOngoing(true);
        Notification notification = builder.getNotification();
        // 设置通知的默认灯光效果
        notification.defaults = Notification.DEFAULT_LIGHTS;
        // 设置通知点击后自动取消
        notification.flags = Notification.FLAG_AUTO_CANCEL;

        // 发送通知
        mNotifiManager.notify(GTASK_SYNC_NOTIFICATION_ID, notification);
    }

    /**
     * 在后台线程中执行同步操作
     *
     * @param unused 未使用的参数
     * @return 同步操作的结果状态码
     */
    @Override
    protected Integer doInBackground(Void... unused) {
        // 发布登录进度消息
        publishProgess(mContext.getString(R.string.sync_progress_login, NotesPreferenceActivity
                .getSyncAccountName(mContext)));
        // 执行同步操作并返回结果
        return mTaskManager.sync(mContext, this);
    }

    /**
     * 在UI线程中更新同步进度
     *
     * @param progress 进度消息数组
     */
    @Override
    protected void onProgressUpdate(String... progress) {
        // 显示同步中的通知
        showNotification(R.string.ticker_syncing, progress[0]);
        if (mContext instanceof GTaskSyncService) {
            // 如果上下文是GTaskSyncService，发送广播
            ((GTaskSyncService) mContext).sendBroadcast(progress[0]);
        }
    }

    /**
     * 在同步操作完成后执行的操作
     *
     * @param result 同步操作的结果状态码
     */
    @Override
    protected void onPostExecute(Integer result) {
        if (result == GTaskManager.STATE_SUCCESS) {
            // 同步成功，显示成功通知
            showNotification(R.string.ticker_success, mContext.getString(
                    R.string.success_sync_account, mTaskManager.getSyncAccount()));
            // 记录最后同步时间
            NotesPreferenceActivity.setLastSyncTime(mContext, System.currentTimeMillis());
        } else if (result == GTaskManager.STATE_NETWORK_ERROR) {
            // 网络错误，显示网络错误通知
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_network));
        } else if (result == GTaskManager.STATE_INTERNAL_ERROR) {
            // 内部错误，显示内部错误通知
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_internal));
        } else if (result == GTaskManager.STATE_SYNC_CANCELLED) {
            // 同步取消，显示取消通知
            showNotification(R.string.ticker_cancel, mContext
                    .getString(R.string.error_sync_cancelled));
        }
        if (mOnCompleteListener != null) {
            // 启动新线程调用完成监听器的方法
            new Thread(new Runnable() {
                @Override
                public void run() {
                    mOnCompleteListener.onComplete();
                }
            }).start();
        }
    }
}